<!-- Forces Info -->
<div class="container p-4">
    <h3>Forces:</h3>
    <ul>
        <li>
            <b>Maximum site forces (vector norm):</b> {{ calculation.site_force_norm_max }}
        </li>
        <li>
            <b>Site forces (vector norm):</b> {{ calculation.site_forces_norm }}
        </li>
        <li>
            <b>Site forces per atom (vector norm):</b> {{ calculation.site_forces_norm_per_atom }}
        </li>
        <li>
            <b>Lattice stress matrix:</b> {{ calculation.lattice_stress }}
        </li>
        <li>
            <b>Lattice stress (vector norm):</b> {{ calculation.lattice_stress_norm }}
        </li>
        <li>
            <b>Lattice stress per atom (vector norm):</b> {{ calculation.lattice_stress_norm_per_atom }}
        </li>
    </ul>
</div>
