{% extends "core_components/site_base.html" %}
{% block tabtitle %}Simmate | Profile{% endblock %}
{% block banner %}
    {% include "core_components/header.html" %}
{% endblock %}
{% block body %}
    <section class="jumbotron text-center">
        <div class="container py-5">
            <h1>Welcome {{ user.username }}!</h1>
            <p class="lead text-muted">
                The <b>Profile</b> page is still under development and is therefore empty at the moment.
                In the future, this will be a place where you can view your "favorite"
                materials as well as track calculations that you've requested. For now,
                switch to the <b>Home</b> page to explore materials.
            </p>
        </div>
    </section>
{% endblock %}
