# Generated by Django 4.2.2 on 2023-07-21 16:22

from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="WorkItem",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(auto_now_add=True, db_index=True, null=True),
                ),
                (
                    "updated_at",
                    models.DateTimeField(auto_now=True, db_index=True, null=True),
                ),
                ("tags", models.JSONField(default=list)),
                (
                    "status",
                    models.CharField(
                        choices=[
                            ("P", "Pending"),
                            ("R", "Running"),
                            ("C", "Cancelled"),
                            ("E", "Errored"),
                            ("F", "Finished"),
                        ],
                        default="P",
                        max_length=1,
                    ),
                ),
                ("command_not_found_failures", models.IntegerField(default=0)),
                ("fxn", models.BinaryField()),
                ("args", models.BinaryField(default=b"\x80\x05]\x94.")),
                ("kwargs", models.BinaryField(default=b"\x80\x05}\x94.")),
                ("result_binary", models.BinaryField(blank=True, null=True)),
            ],
        ),
    ]
