{% include "core_components/base_data_types/calculation.html" %}
<div class="container p-4">
    <h3>Search Settings:</h3>
    <ul>
        <li>
            <b>Composition:</b> {{ calculation.composition }}
        </li>
        <li>
            <b>Subworkflow name:</b> {{ calculation.subworkflow_name }}
        </li>
        <li>
            <b>Subworkflow parameters:</b> {{ calculation.subworkflow_kwargs }}
        </li>
        <li>
            <b>Fitness field:</b> {{ calculation.fitness_field }}
        </li>
        <li>
            <b>Minimum 'exact' structures:</b> {{ calculation.min_structures_exact }}
        </li>
        <li>
            <b>Maximum structures:</b> {{ calculation.max_structures }}
        </li>
        <li>
            <b>Best survival cutoff:</b> {{ calculation.best_survival_cutoff }}
        </li>
        <li>
            <b>First generation size:</b> {{ calculation.nfirst_generation }}
        </li>
        <li>
            <b>Total steady-state count:</b> {{ calculation.nsteadystate }}
        </li>
        <li>
            <b>Convergence cutoff:</b> {{ calculation.convergence_cutoff }} eV
        </li>
        <li>
            <b>Parent selection method:</b> {{ calculation.selector_name }}
        </li>
        <li>
            <b>Parent selection parameters:</b> {{ calculation.selector_kwargs }}
        </li>
        <li>
            <b>Validation method:</b> {{ calculation.validator_name }} eV
        </li>
        <li>
            <b>Validation parameters:</b> {{ calculation.validator_kwargs }}
        </li>
        <li>
            <b>Single-shot sources:</b> {{ calculation.singleshot_sources }}
        </li>
        <li>
            <b>Sleep step (s):</b> {{ calculation.sleep_step }}
        </li>
    </ul>
</div>
<div class="container p-4">
    <h3>Plots:</h3>
    {{ calculation.get_fitness_convergence_html_div | safe }}
    {{ calculation.get_fitness_distribution_html_div | safe }}
    {{ calculation.get_staged_series_convergence_html_div | safe }}
    {{ calculation.get_staged_series_histogram_html_div | safe }}
    {{ calculation.get_staged_series_times_html_div | safe }}
    {{ calculation.get_subworkflow_times_html_div | safe }}
</div>
