{% extends "core_components/site_base.html" %}
{% block tabtitle %}Simmate{% endblock %}
{% block banner %}
    {% include "core_components/header.html" %}
{% endblock %}
{% block body %}
    <!-- Header -->
    <section class="pt-5 jumbotron text-center">
        <div class="container">
            <h1 class="jumbotron-heading">Apps</h1>
            <p class="lead text-muted">Additional apps, plugins, & dashboards made with Simmate.</p>
        </div>
    </section>
    <!-- List of Cards -->
    <div class="row">
        {% for extra_app in extra_apps %}
            <div class="col-md-4">
                <div class="card border-primary border" style="height: 90%">
                    <div class="card-body">
                        <h5 class="card-title text-primary">{{ extra_app.verbose_name }}</h5>
                        <p class="card-text">{{ extra_app.description_short }}</p>
                        <a role="button"
                           class="btn btn-primary btn-sm"
                           href="{{ extra_app.short_name }}">Explore</a>
                    </div>
                </div>
            </div>
        {% endfor %}
    </div>
{% endblock %}
