[unicorn\:error\:invalid] {
    border: 1px solid red !important;
}
[unicorn\:error\:required] {
    border: 1px solid red !important;
}
[unicorn\:error\:max_length] {
    border: 1px solid red !important;
}
[unicorn\:error\:min_length] {
    border: 1px solid red !important;
}
.error {
    color: red;
}
.unicorn-errors {
    background-color: #feecf0;
    color: #cc0f35;
    border-radius: 4px;
    position: relative;
    padding: .25rem 2.5rem 1.25rem 1.5rem;
    margin-bottom: 2rem;
}