{% load crispy_forms_tags %}
<div class="card mb-0">
    <div class="card-header" id="headingThermoFilters">
        <h5 class="m-0">
            <a class="custom-accordion-title d-block pt-2 pb-2"
               data-bs-toggle="collapse"
               href="#collapseThermoFilters"
               aria-expanded="true"
               aria-controls="collapseThermoFilters">
                By Thermodynamic Info:
                <i class="mdi mdi-chevron-down accordion-arrow"></i>
            </a>
        </h5>
    </div>
    <div id="collapseThermoFilters"
         class="collapse"
         aria-labelledby="headingThermoFilters"
         data-bs-parent="#accordionExample">
        <div class="card-body">
            {{ form.energy__range | as_crispy_field }}
            {{ form.energy_per_atom__range | as_crispy_field }}
            {{ form.energy_above_hull__range | as_crispy_field }}
            {{ form.is_stable | as_crispy_field }}
            {{ form.formation_energy__range | as_crispy_field }}
            {{ form.formation_energy__range | as_crispy_field }}
        </div>
    </div>
</div>
