{% load crispy_forms_tags %}
<div class="card mb-0">
    <div class="card-header" id="headingForcesFilters">
        <h5 class="m-0">
            <a class="custom-accordion-title d-block pt-2 pb-2"
               data-bs-toggle="collapse"
               href="#collapseForcesFilters"
               aria-expanded="true"
               aria-controls="collapseForcesFilters">
                By Forces Info:
                <i class="mdi mdi-chevron-down accordion-arrow"></i>
            </a>
        </h5>
    </div>
    <div id="collapseForcesFilters"
         class="collapse"
         aria-labelledby="headingForcesFilters"
         data-bs-parent="#accordionExample">
        <div class="card-body">
            {{ form.site_force_norm_max__range | as_crispy_field }}
            {{ form.site_forces_norm_per_atom__range | as_crispy_field }}
            {{ form.lattice_stress_norm__range | as_crispy_field }}
            {{ form.lattice_stress_norm_per_atom__range | as_crispy_field }}
        </div>
    </div>
</div>
