# Generated by Django 4.2.2 on 2023-08-07 23:16

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("data_explorer", "0001_initial"),
    ]

    operations = [
        migrations.AlterField(
            model_name="aflowprototype",
            name="id",
            field=models.AutoField(primary_key=True, serialize=False),
        ),
    ]
