# -*- coding: utf-8 -*-

from django.apps import AppConfig


class BadelfConfig(AppConfig):
    name = "simmate.apps.badelf"


class BaderConfig(AppConfig):
    name = "simmate.apps.bader"


class CleaseConfig(AppConfig):
    name = "simmate.apps.clease"


class EvolutionConfig(AppConfig):
    name = "simmate.apps.evolution"


class MaterialsProjectConfig(AppConfig):
    name = "simmate.apps.materials_project"


class VaspConfig(AppConfig):
    name = "simmate.apps.vasp"


class QuantumEspressoConfig(AppConfig):
    name = "simmate.apps.quantum_espresso"


class WarrenLabConfig(AppConfig):
    name = "simmate.apps.warren_lab"
